package com.slibra.web.controller.business;

import java.util.Date;
import java.util.List;
import java.util.Objects;
import javax.servlet.http.HttpServletResponse;

import com.slibra.common.utils.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.slibra.common.annotation.Log;
import com.slibra.common.core.controller.BaseController;
import com.slibra.common.core.domain.AjaxResult;
import com.slibra.common.enums.BusinessType;
import com.slibra.business.domain.TXinyiWarningRecord;
import com.slibra.business.service.ITXinyiWarningRecordService;
import com.slibra.common.utils.poi.ExcelUtil;
import com.slibra.common.core.page.TableDataInfo;

import static com.slibra.common.constant.MyConstants.MAX_COUNT;
import static com.slibra.common.constant.MyConstants.MAX_COUNT_STR;

/**
 * 信义告警记录Controller
 * 
 * @author slibra
 * @date 2024-05-17
 */
@Slf4j
@RestController
@RequestMapping("/business/record")
public class TXinyiWarningRecordController extends BaseController
{
    @Autowired
    private ITXinyiWarningRecordService tXinyiWarningRecordService;

    /**
     * 查询信义告警记录列表
     */
    @PreAuthorize("@ss.hasPermi('business:record:list')")
    @GetMapping("/list")
    public TableDataInfo list(TXinyiWarningRecord tXinyiWarningRecord)
    {
        startPage();
        List<TXinyiWarningRecord> list = tXinyiWarningRecordService.selectTXinyiWarningRecordList(tXinyiWarningRecord);
        handleWarningCounts(list);
        return getDataTable(list);
    }

    /**
     * 导出信义告警记录列表
     */
    @PreAuthorize("@ss.hasPermi('business:record:export')")
    @Log(title = "信义告警记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TXinyiWarningRecord tXinyiWarningRecord)
    {
        List<TXinyiWarningRecord> list = tXinyiWarningRecordService.selectTXinyiWarningRecordList(tXinyiWarningRecord);
        handleWarningCounts(list);
        ExcelUtil<TXinyiWarningRecord> util = new ExcelUtil<TXinyiWarningRecord>(TXinyiWarningRecord.class);
        util.exportExcel(response, list, "信义告警记录数据");
    }

    /**
     * 获取信义告警记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:record:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(tXinyiWarningRecordService.selectTXinyiWarningRecordById(id));
    }

    /**
     * 新增信义告警记录
     */
    @PreAuthorize("@ss.hasPermi('business:record:add')")
    @Log(title = "信义告警记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TXinyiWarningRecord tXinyiWarningRecord)
    {
        tXinyiWarningRecord.setCreateBy(getUsername());
        return toAjax(tXinyiWarningRecordService.insertTXinyiWarningRecord(tXinyiWarningRecord));
    }

    /**
     * 修改信义告警记录
     */
    @PreAuthorize("@ss.hasPermi('business:record:edit')")
    @Log(title = "信义告警记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TXinyiWarningRecord tXinyiWarningRecord)
    {
        tXinyiWarningRecord.setUpdateBy(getUsername());
        return toAjax(tXinyiWarningRecordService.updateTXinyiWarningRecord(tXinyiWarningRecord));
    }

    /**
     * 删除信义告警记录
     */
    @PreAuthorize("@ss.hasPermi('business:record:remove')")
    @Log(title = "信义告警记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tXinyiWarningRecordService.deleteTXinyiWarningRecordByIds(ids));
    }


    private void handleWarningCounts(List<TXinyiWarningRecord> tXinyiWarningRecords) {
        if(!CollectionUtils.isEmpty(tXinyiWarningRecords)){
            for (TXinyiWarningRecord tXinyiWarningRecord : tXinyiWarningRecords) {
                Date endDate = null;
                //判断类型 如果结束了 截止时间就是告警截止时间；  如果正在报警 截止时间就是当前时间
                if(0 == tXinyiWarningRecord.getStatus() || 3 == tXinyiWarningRecord.getStatus())
                    endDate = DateUtils.getNowDate();
                else
                    endDate = tXinyiWarningRecord.getOffTime();
                //防止 之前程序处理错误  理论上不再报警了，一定有截止时间的
                if(Objects.isNull(endDate)){
                    log.error("xxxxxxxxx：之前处理告警结束的逻辑异常，没有更新截止时间");
                    endDate = DateUtils.getNowDate();
                }
                long counts = DateUtils.differentHoursByLocalDateTime(tXinyiWarningRecord.getCreateTime(), endDate) + 1;
                tXinyiWarningRecord.setCounts(counts > MAX_COUNT ? MAX_COUNT_STR : String.valueOf(counts));
            }
        }
    }
}
