package com.slibra.business.service.impl;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

import com.alibaba.fastjson2.JSON;
import com.slibra.business.domain.TXinyiCarbonExtra;
import com.slibra.business.mapper.TXinyiCarbonExtraMapper;
import com.slibra.business.res.CarbonSmartQueryDB;
import com.slibra.business.res.ChartBasic;
import com.slibra.common.DecimalUtils;
import com.slibra.common.core.domain.entity.SysDictData;
import com.slibra.common.enums.BusinessEnum;
import com.slibra.common.exception.ServiceException;
import com.slibra.common.utils.DateUtils;
import com.slibra.common.utils.SecurityUtils;
import com.slibra.common.utils.StringUtils;
import com.slibra.system.mapper.SysDictDataMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.slibra.business.mapper.TXinyiCarbonMajorMapper;
import com.slibra.business.domain.TXinyiCarbonMajor;
import com.slibra.business.service.ITXinyiCarbonMajorService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import static com.slibra.common.constant.MyConstants.*;

/**
 * 碳排放主Service业务层处理
 * 
 * @author slibra
 * @date 2024-08-24
 */
@Service
public class TXinyiCarbonMajorServiceImpl implements ITXinyiCarbonMajorService
{
    private static final Logger log = LoggerFactory.getLogger(TXinyiCarbonMajorServiceImpl.class);
    @Autowired
    private TXinyiCarbonMajorMapper tXinyiCarbonMajorMapper;

    @Autowired
    private SysDictDataMapper sysDictDataMapper;

    @Autowired
    private TXinyiCarbonExtraMapper xinyiCarbonExtraMapper;

    /**
     * 查询碳排放主
     * 
     * @param id 碳排放主主键
     * @return 碳排放主
     */
    @Override
    public TXinyiCarbonMajor selectTXinyiCarbonMajorById(Long id)
    {
        return tXinyiCarbonMajorMapper.selectTXinyiCarbonMajorById(id);
    }

    /**
     * 查询碳排放主列表
     * 
     * @param tXinyiCarbonMajor 碳排放主
     * @return 碳排放主
     */
    @Override
    public List<TXinyiCarbonMajor> selectTXinyiCarbonMajorList(TXinyiCarbonMajor tXinyiCarbonMajor)
    {
        return tXinyiCarbonMajorMapper.selectTXinyiCarbonMajorList(tXinyiCarbonMajor);
    }

    /**
     * 新增碳排放主
     * 
     * @param tXinyiCarbonMajor 碳排放主
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int insertTXinyiCarbonMajor(TXinyiCarbonMajor tXinyiCarbonMajor)
    {
        Date nowDate = DateUtils.getNowDate();
        String username = SecurityUtils.getUsername();
        tXinyiCarbonMajor.setCreateTime(nowDate);
        tXinyiCarbonMajor.setCreateBy(username);
        String addYearMonth = tXinyiCarbonMajor.getAddYearMonth();
        if(StringUtils.isBlank(addYearMonth))
            throw new ServiceException("请输入核算时间");
        List<TXinyiCarbonMajor> tXinyiCarbonMajors = this.tXinyiCarbonMajorMapper.selectTXinyiCarbonMajorList(TXinyiCarbonMajor.builder().addYearMonth(addYearMonth).build());
        if(!CollectionUtils.isEmpty(tXinyiCarbonMajors))
            throw new ServiceException("对应的核算时间已经填写过了，不可以再次填写");
        tXinyiCarbonMajor.setAddYear(addYearMonth.substring(0, 4));
        //填报日期（只要年月日）
        tXinyiCarbonMajor.setRemark(DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, nowDate));
        int i = tXinyiCarbonMajorMapper.insertTXinyiCarbonMajor(tXinyiCarbonMajor);
        Long id = tXinyiCarbonMajor.getId();
        log.info("要保存的主表的碳核查智能体的记录ID为{}", id);
        //查看污水厂所在区域电力行业的平均CO2当量排放因子（kg CO2-eq/(kW·h)）
        Long provinceId = tXinyiCarbonMajor.getProvinceId();
        BigDecimal yz = null;
        if(!Objects.isNull(provinceId)) {
            SysDictData sysDictData = this.sysDictDataMapper.selectDictDataById(provinceId);
            if (Objects.nonNull(sysDictData)) {
                String dictValue = sysDictData.getDictValue();
                if (StringUtils.isNotBlank(dictValue)) {
                    yz = new BigDecimal(dictValue);
                }
            }
        }

        //处理关联表
        List<TXinyiCarbonExtra> extraList = tXinyiCarbonMajor.getExtraList();
        if(!CollectionUtils.isEmpty(extraList)){
            for (TXinyiCarbonExtra tXinyiCarbonExtra : extraList) {
                tXinyiCarbonExtra.setMajorId(id);
                tXinyiCarbonExtra.setVal(this.calculate(tXinyiCarbonExtra, yz));//需要单独计算
                tXinyiCarbonExtra.setCreateTime(nowDate);
                tXinyiCarbonExtra.setCreateBy(username);
                this.xinyiCarbonExtraMapper.insertTXinyiCarbonExtra(tXinyiCarbonExtra);
            }
        }
        TXinyiCarbonMajor tXinyiCarbonMajorUpdate = new TXinyiCarbonMajor();
        tXinyiCarbonMajorUpdate.setId(id);
        //更新部分字段（计算相关：多条数据的）
        Optional<BigDecimal> reduce0 = extraList.stream().filter(v -> v.getType() == BusinessEnum.CarbonExtraTeypEnum.ZERO.getCode()).map(TXinyiCarbonExtra::getVal).reduce(BigDecimal::add);
        reduce0.ifPresent(tXinyiCarbonMajorUpdate::setWjtykuCo2);
        Optional<BigDecimal> reduce1 = extraList.stream().filter(v -> v.getType() == BusinessEnum.CarbonExtraTeypEnum.ONE.getCode()).map(TXinyiCarbonExtra::getVal).reduce(BigDecimal::add);
        reduce1.ifPresent(tXinyiCarbonMajorUpdate::setGdsnyrlCo2);
        Optional<BigDecimal> reduce2 = extraList.stream().filter(v -> v.getType() == BusinessEnum.CarbonExtraTeypEnum.TWO.getCode()).map(TXinyiCarbonExtra::getVal).reduce(BigDecimal::add);
        reduce2.ifPresent(tXinyiCarbonMajorUpdate::setWscljdyjxhCo2);
        Optional<BigDecimal> reduce3 = extraList.stream().filter(v -> v.getType() == BusinessEnum.CarbonExtraTeypEnum.THREE.getCode()).map(TXinyiCarbonExtra::getVal).reduce(BigDecimal::add);
        reduce3.ifPresent(tXinyiCarbonMajorUpdate::setRbjsTTL);
        Optional<BigDecimal> reduce4 = extraList.stream().filter(v -> v.getType() == BusinessEnum.CarbonExtraTeypEnum.FOUR.getCode()).map(TXinyiCarbonExtra::getVal).reduce(BigDecimal::add);
        reduce4.ifPresent(tXinyiCarbonMajorUpdate::setWnyyxhzqlyTTL);
        Optional<BigDecimal> reduce5 = extraList.stream().filter(v -> v.getType() == BusinessEnum.CarbonExtraTeypEnum.FIVE.getCode()).map(TXinyiCarbonExtra::getVal).reduce(BigDecimal::add);
        reduce5.ifPresent(tXinyiCarbonMajorUpdate::setWnfshrjnlyTTL);

        //处理计算信息
        //------------污水处理------------
        BigDecimal wsclTotalCo2 = BigDecimal.ZERO;
        BigDecimal wsHsytkhClsl = tXinyiCarbonMajor.getWsHsytkhClsl();
        BigDecimal wsHsytkhJsCod = tXinyiCarbonMajor.getWsHsytkhJsCod();
        BigDecimal wsHsytkhCsCod = tXinyiCarbonMajor.getWsHsytkhCsCod();
        BigDecimal wsHsytkhCo2 = null;
        if(!Objects.isNull(wsHsytkhClsl) && !Objects.isNull(wsHsytkhJsCod) && !Objects.isNull(wsHsytkhCsCod)){
            wsHsytkhCo2 = DecimalUtils.getNLengthHalfUp((wsHsytkhJsCod.subtract(wsHsytkhCsCod)).abs().multiply(wsHsytkhClsl).multiply(BigDecimal_0_56).multiply(BigDecimal_0_1).divide(BigDecimal_1000000, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
            tXinyiCarbonMajorUpdate.setWsHsytkhCo2(wsHsytkhCo2);
        }
        BigDecimal wsSjclCh4Clsl = tXinyiCarbonMajor.getWsSjclCh4Clsl();
        BigDecimal wsSjclCh4JsCod = tXinyiCarbonMajor.getWsSjclCh4JsCod();
        BigDecimal wsSjclCh4CsCod = tXinyiCarbonMajor.getWsSjclCh4CsCod();
        BigDecimal wsSjclCh4Pfyz = tXinyiCarbonMajor.getWsSjclCh4Pfyz();
        BigDecimal wsSjclCh4Tsb = tXinyiCarbonMajor.getWsSjclCh4Tsb();
        BigDecimal wsSjclCh4Csc = tXinyiCarbonMajor.getWsSjclCh4Csc();
        BigDecimal wsSjclCh4Co2 = null;
        if(!Objects.isNull(wsSjclCh4Clsl) && !Objects.isNull(wsSjclCh4JsCod) && !Objects.isNull(wsSjclCh4CsCod)){
            if(Objects.isNull(wsSjclCh4Pfyz)){
                wsSjclCh4Pfyz = BigDecimal_0_0075;
            }
            //判断是否有实测
            if(!Objects.isNull(wsSjclCh4Tsb) && !Objects.isNull(wsSjclCh4Csc)){//有实测
                wsSjclCh4Co2 = DecimalUtils.getNLengthHalfUp(((wsSjclCh4JsCod.subtract(wsSjclCh4CsCod)).abs().multiply(wsSjclCh4Clsl).multiply(wsSjclCh4Pfyz).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP).add(wsSjclCh4Tsb).add(wsSjclCh4Csc)).multiply(BigDecimal_28).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
                tXinyiCarbonMajorUpdate.setWsSjclCh4Co2(wsSjclCh4Co2);
            }else{//没有实测
                wsSjclCh4Co2 = DecimalUtils.getNLengthHalfUp((wsSjclCh4JsCod.subtract(wsSjclCh4CsCod)).abs().multiply(wsSjclCh4Clsl).multiply(wsSjclCh4Pfyz).multiply((BigDecimal_1.add(BigDecimal_0_2))).multiply(BigDecimal_28).divide(BigDecimal_1000000, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
                tXinyiCarbonMajorUpdate.setWsSjclCh4Co2(wsSjclCh4Co2);
            }
        }

        BigDecimal wsTdN2oClsl = tXinyiCarbonMajor.getWsTdN2oClsl();
        BigDecimal wsTdN2oJsTn = tXinyiCarbonMajor.getWsTdN2oJsTn();
        BigDecimal wsTdN2oCsTn = tXinyiCarbonMajor.getWsTdN2oCsTn();
        BigDecimal wsTdN2oPfyz = tXinyiCarbonMajor.getWsTdN2oPfyz();
        BigDecimal wsTdN2oCo2 = null;
        if(!Objects.isNull(wsTdN2oClsl) && !Objects.isNull(wsTdN2oJsTn) && !Objects.isNull(wsTdN2oCsTn)){
            if(Objects.isNull(wsTdN2oPfyz)){
                wsTdN2oPfyz = BigDecimal_0_01;
            }
            wsTdN2oCo2 = DecimalUtils.getNLengthHalfUp((wsTdN2oJsTn.subtract(wsTdN2oCsTn)).abs().multiply(wsTdN2oClsl).multiply(wsTdN2oPfyz).multiply(BigDecimal_265).multiply(BigDecimal_44).divide(BigDecimal_28, NUMBER_SCALE_6, RoundingMode.HALF_UP).divide(BigDecimal_1000000, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
            tXinyiCarbonMajorUpdate.setWsTdN2oCo2(wsTdN2oCo2);
        }

        //处理 污水处理碳排放量
        if(!Objects.isNull(wsHsytkhCo2)){
            wsclTotalCo2 = wsclTotalCo2.add(wsHsytkhCo2);
        }
        if(!Objects.isNull(wsSjclCh4Co2)){
            wsclTotalCo2 = wsclTotalCo2.add(wsSjclCh4Co2);
        }
        if(!Objects.isNull(wsTdN2oCo2)){
            wsclTotalCo2 = wsclTotalCo2.add(wsTdN2oCo2);
        }
        //一对多的数据
        BigDecimal wjtykuCo2 = tXinyiCarbonMajorUpdate.getWjtykuCo2();
        if(!Objects.isNull(wjtykuCo2)){
            wsclTotalCo2 = wsclTotalCo2.add(wjtykuCo2);
        }
        tXinyiCarbonMajorUpdate.setWsclTotalCo2(BigDecimal.ZERO.compareTo(wsclTotalCo2) == 0 ? null : wsclTotalCo2);


        //------------能源、药剂排放------------
        //耗电相关
        BigDecimal nyyjTotalCo2 = BigDecimal.ZERO;
        BigDecimal nyyjDlxhZhdlCo2 = null;
        BigDecimal nyyjDlxhTotal = BigDecimal.ZERO;
        BigDecimal nyyjDlxhZhdl = null;
        if(!Objects.isNull(yz)){
            //总耗电
            nyyjDlxhZhdl = tXinyiCarbonMajor.getNyyjDlxhZhdl();
            if(!Objects.isNull(nyyjDlxhZhdl)){
                nyyjDlxhZhdlCo2 = DecimalUtils.getNLengthHalfUp(nyyjDlxhZhdl.multiply(yz).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
                tXinyiCarbonMajorUpdate.setNyyjDlxhZhdlCo2(nyyjDlxhZhdlCo2);
            }

            //进水泵房
            BigDecimal nyyjDlxhJsbf = tXinyiCarbonMajor.getNyyjDlxhJsbf();
            if(!Objects.isNull(nyyjDlxhJsbf)){
                tXinyiCarbonMajorUpdate.setNyyjDlxhJsbfCo2(nyyjDlxhJsbf.multiply(yz).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP));
                nyyjDlxhTotal = nyyjDlxhTotal.add(nyyjDlxhJsbf);
            }

            //鼓风机房
            BigDecimal nyyjDlxhGfjf = tXinyiCarbonMajor.getNyyjDlxhGfjf();
            if(!Objects.isNull(nyyjDlxhGfjf)){
                tXinyiCarbonMajorUpdate.setNyyjDlxhGfjfCo2(nyyjDlxhGfjf.multiply(yz).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP));
                nyyjDlxhTotal = nyyjDlxhTotal.add(nyyjDlxhGfjf);
            }

            //脱水机房
            BigDecimal nyyjDlxhTsjf = tXinyiCarbonMajor.getNyyjDlxhTsjf();
            if(!Objects.isNull(nyyjDlxhTsjf)){
                tXinyiCarbonMajorUpdate.setNyyjDlxhTsjfCo2(nyyjDlxhTsjf.multiply(yz).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP));
                nyyjDlxhTotal = nyyjDlxhTotal.add(nyyjDlxhTsjf);
            }

            //消毒间
            BigDecimal nyyjDlxhXdj = tXinyiCarbonMajor.getNyyjDlxhXdj();
            if(!Objects.isNull(nyyjDlxhXdj)){
                tXinyiCarbonMajorUpdate.setNyyjDlxhXdjCo2(nyyjDlxhXdj.multiply(yz).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP));
                nyyjDlxhTotal = nyyjDlxhTotal.add(nyyjDlxhXdj);
            }

            //深度处理用电
            BigDecimal nyyjDlxhSdcl = tXinyiCarbonMajor.getNyyjDlxhSdcl();
            if(!Objects.isNull(nyyjDlxhSdcl)){
                tXinyiCarbonMajorUpdate.setNyyjDlxhSdclCo2(nyyjDlxhSdcl.multiply(yz).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP));
                nyyjDlxhTotal = nyyjDlxhTotal.add(nyyjDlxhSdcl);
            }

            //污泥处置
            BigDecimal nyyjDlxhWncz = tXinyiCarbonMajor.getNyyjDlxhWncz();
            if(!Objects.isNull(nyyjDlxhWncz)){
                tXinyiCarbonMajorUpdate.setNyyjDlxhWnczCo2(nyyjDlxhWncz.multiply(yz).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP));
                nyyjDlxhTotal = nyyjDlxhTotal.add(nyyjDlxhWncz);
            }

            //生活区
            BigDecimal nyyjDlxhShq = tXinyiCarbonMajor.getNyyjDlxhShq();
            if(!Objects.isNull(nyyjDlxhShq)){
                tXinyiCarbonMajorUpdate.setNyyjDlxhShqCo2(nyyjDlxhShq.multiply(yz).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP));
                nyyjDlxhTotal = nyyjDlxhTotal.add(nyyjDlxhShq);
            }

            //其他耗电区
            BigDecimal nyyjDlxhQt = tXinyiCarbonMajor.getNyyjDlxhQt();
            if(!Objects.isNull(nyyjDlxhQt)){
                tXinyiCarbonMajorUpdate.setNyyjDlxhQtCo2(nyyjDlxhQt.multiply(yz).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP));
                nyyjDlxhTotal = nyyjDlxhTotal.add(nyyjDlxhQt);
            }
        }

        //2024年09月05日14:09:26 增加校验，各个分项的值累加不能超过总的
        if(!Objects.isNull(nyyjDlxhZhdl) && nyyjDlxhTotal.compareTo(nyyjDlxhZhdl) > 0){
            log.error("各分项耗电量之和不能超过总耗电量！");
            throw new ServiceException("各分项耗电量之和不能超过总耗电量！");
        }

        //能源、药剂碳排-净购入热力消耗产生的碳排放量-燃料消耗产生的碳排放量
        BigDecimal nyyjGrrlZq = tXinyiCarbonMajor.getNyyjGrrlZq();
        BigDecimal nyyjGrrlDyhz = tXinyiCarbonMajor.getNyyjGrrlDyhz();
        BigDecimal nyyjGrrlRl = tXinyiCarbonMajor.getNyyjGrrlRl();
        BigDecimal nyyjGrrlCo2 = null;
        //2024年08月27日17:21:58  计算的时候，不用nyyjGrrlRl了，用前面的2个值，计算出来热力nyyjGrrlRl
        if(!Objects.isNull(nyyjGrrlZq) && !Objects.isNull(nyyjGrrlDyhz)){
            nyyjGrrlCo2 = DecimalUtils.getNLengthHalfUp((nyyjGrrlDyhz.subtract(BigDecimal_83_74)).abs().multiply(nyyjGrrlZq).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP).multiply(BigDecimal_110000).divide(BigDecimal_1000000, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
            tXinyiCarbonMajorUpdate.setNyyjGrrlCo2(nyyjGrrlCo2);
        }

        //处理能源、药剂碳总碳排量
        if(!Objects.isNull(nyyjDlxhZhdlCo2)){
            nyyjTotalCo2 = nyyjTotalCo2.add(nyyjDlxhZhdlCo2);
        }
        if(!Objects.isNull(nyyjGrrlCo2)){
            nyyjTotalCo2 = nyyjTotalCo2.add(nyyjGrrlCo2);
        }
        //处理一对多的两条数据
        BigDecimal gdsnyrlCo2 = tXinyiCarbonMajorUpdate.getGdsnyrlCo2();
        if(!Objects.isNull(gdsnyrlCo2)){
            nyyjTotalCo2 = nyyjTotalCo2.add(gdsnyrlCo2);
        }
        BigDecimal wscljdyjxhCo2 = tXinyiCarbonMajorUpdate.getWscljdyjxhCo2();
        if(!Objects.isNull(wscljdyjxhCo2)){
            nyyjTotalCo2 = nyyjTotalCo2.add(wscljdyjxhCo2);
        }
        //总的
        tXinyiCarbonMajorUpdate.setNyyjTotalCo2(BigDecimal.ZERO.compareTo(nyyjTotalCo2) == 0 ? null : nyyjTotalCo2);


        //------------污泥处理------------
        BigDecimal wnclTotalCo2 = BigDecimal.ZERO;
        //污泥处理-污泥厌氧消化沼气收集管路无意泄露的CH₄或沼气火炬燃烧不充分导致的碳排放-污泥厌氧消化过程排放的CH₄的CO₂当量（t CO₂-eq/月）
        BigDecimal wnclYyzqZqcl = tXinyiCarbonMajor.getWnclYyzqZqcl();
        BigDecimal wnclYyzqChZlfs = tXinyiCarbonMajor.getWnclYyzqChZlfs();
        BigDecimal wnclYyzqCo2 = null;
        if(!Objects.isNull(wnclYyzqZqcl) && !Objects.isNull(wnclYyzqChZlfs)){
            wnclYyzqCo2 = DecimalUtils.getNLengthHalfUp(wnclYyzqZqcl.multiply(wnclYyzqChZlfs).multiply(BigDecimal_0_05).multiply(BigDecimal_28).multiply(BigDecimal_16).divide(BigDecimal_22_4, NUMBER_SCALE_6, RoundingMode.HALF_UP).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP).divide(BigDecimal_100, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
            tXinyiCarbonMajorUpdate.setWnclYyzqCo2(wnclYyzqCo2);
        }

        //污泥处理-单独处理污泥厌氧消化过程产生的碳排放量-单独处理污泥厌氧消化产生沼液的CO₂当量（t CO₂-eq/月）
        BigDecimal wnclDdclZycll = tXinyiCarbonMajor.getWnclDdclZycll();
        BigDecimal wnclDdclClqnd = tXinyiCarbonMajor.getWnclDdclClqnd();
        BigDecimal wnclDdclClhnd = tXinyiCarbonMajor.getWnclDdclClhnd();
        BigDecimal wnclDdclPfyz = tXinyiCarbonMajor.getWnclDdclPfyz();
        BigDecimal wnclDdclCo2 = null;
        if(!Objects.isNull(wnclDdclZycll) && !Objects.isNull(wnclDdclClqnd) && !Objects.isNull(wnclDdclClhnd)){
            if(Objects.isNull(wnclDdclPfyz)){
                wnclDdclPfyz = BigDecimal_0_01;
            }
            wnclDdclCo2 = DecimalUtils.getNLengthHalfUp((wnclDdclClqnd.subtract(wnclDdclClhnd)).abs().multiply(wnclDdclZycll).multiply(wnclDdclPfyz).multiply(BigDecimal_265).multiply(BigDecimal_44).divide(BigDecimal_28, NUMBER_SCALE_6, RoundingMode.HALF_UP).divide(BigDecimal_1000000, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
            tXinyiCarbonMajorUpdate.setWnclDdclCo2(wnclDdclCo2);
        }

        //污泥处理-污泥好氧发酵过程生化反应的碳排放-污泥好氧发酵过程排放的CH₄的CO₂当量（t CO₂-eq/月）
        //污泥处理-污泥好氧发酵过程生化反应的碳排放-污泥好氧发酵过程排放的N₂O的CO₂当量（t CO₂-eq/月）
        BigDecimal wnclHyfjFjl = tXinyiCarbonMajor.getWnclHyfjFjl();
        BigDecimal wnclHyfjCh4Co2 = null;
        BigDecimal wnclHyfjN2oCo2 = null;
        if(!Objects.isNull(wnclHyfjFjl)){
            wnclHyfjCh4Co2 = DecimalUtils.getNLengthHalfUp(wnclHyfjFjl.multiply(BigDecimal_10).multiply(BigDecimal_28).divide(BigDecimal_1000000, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
            tXinyiCarbonMajorUpdate.setWnclHyfjCh4Co2(wnclHyfjCh4Co2);
            wnclHyfjN2oCo2 = DecimalUtils.getNLengthHalfUp(wnclHyfjFjl.multiply(BigDecimal_0_6).multiply(BigDecimal_265).divide(BigDecimal_1000000, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
            tXinyiCarbonMajorUpdate.setWnclHyfjN2oCo2(wnclHyfjN2oCo2);
        }

        //污泥处理-污泥干化焚烧或协同焚烧过程，化石源碳被氧化产生的化石源CO₂碳排放量-污泥好氧发酵过程排放的CH₄的CO₂当量（t CO₂-eq/月）
        BigDecimal wnclWnfsXhl = tXinyiCarbonMajor.getWnclWnfsXhl();
        BigDecimal wnclWnfsHtbl = tXinyiCarbonMajor.getWnclWnfsHtbl();
        BigDecimal wnclWnfsCo2 = null;

        if(!Objects.isNull(wnclWnfsXhl) && !Objects.isNull(wnclWnfsHtbl)){
            wnclWnfsCo2 = DecimalUtils.getNLengthHalfUp(wnclWnfsXhl.multiply(wnclWnfsHtbl).multiply(BigDecimal_0_12).multiply(BigDecimal_44).divide(BigDecimal_12, NUMBER_SCALE_6, RoundingMode.HALF_UP).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP).divide(BigDecimal_100, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
            tXinyiCarbonMajorUpdate.setWnclWnfsCo2(wnclWnfsCo2);
        }

        //污泥处理-污泥焚烧过程不完全燃烧产生的CH₄和N₂O的CO₂排放当量-污泥焚烧过程排放的CH₄的CO₂当量（t CO₂-eq/月）
        //污泥处理-污泥焚烧过程不完全燃烧产生的CH₄和N₂O的CO₂排放当量-污泥焚烧过程排放的N₂O的CO₂当量（t CO₂-eq/月）
        //逻辑调整：wnclBwqrsCh4Co2的排放因子根据类型获取； wnclBwqrsN2oCo2的排放因子根据判断用户是否填写，没有用默认值
        Integer wnclBwqrsRslx = tXinyiCarbonMajor.getWnclBwqrsRslx();
        BigDecimal wnclBwqrsFsl = tXinyiCarbonMajor.getWnclBwqrsFsl();
        BigDecimal wnclBwqrsPfyzOnlyForN2o = tXinyiCarbonMajor.getWnclBwqrsPfyz();
        BigDecimal wnclBwqrsCh4Co2 = null;
        BigDecimal wnclBwqrsN2oCo2 = null;
        if(!Objects.isNull(wnclBwqrsFsl)){
            BigDecimal wnclBwqrsPfyz = BigDecimal.ZERO;
            if (!Objects.isNull(wnclBwqrsRslx)) {
                if(0 == wnclBwqrsRslx){
                    wnclBwqrsPfyz = BigDecimal_0_06;
                } else if (1 == wnclBwqrsRslx) {
                    wnclBwqrsPfyz = BigDecimal_1_88;
                }else if (2 == wnclBwqrsRslx) {
                    wnclBwqrsPfyz = BigDecimal_0_6;
                }else if (3 == wnclBwqrsRslx) {
                    wnclBwqrsPfyz = BigDecimal_2_37;
                }else if (4 == wnclBwqrsRslx) {
                    wnclBwqrsPfyz = BigDecimal.ZERO;
                }
            } /*else {
                //类型前端必须选，不选默认就是0
                wnclBwqrsPfyz = BigDecimal.ZERO;
            }*/
            wnclBwqrsCh4Co2 = DecimalUtils.getNLengthHalfUp(wnclBwqrsFsl.multiply(wnclBwqrsPfyz).multiply(BigDecimal_28).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
            tXinyiCarbonMajorUpdate.setWnclBwqrsCh4Co2(wnclBwqrsCh4Co2);
            wnclBwqrsN2oCo2 = DecimalUtils.getNLengthHalfUp(wnclBwqrsFsl.multiply(Objects.isNull(wnclBwqrsPfyzOnlyForN2o) ? BigDecimal_0_99 : wnclBwqrsPfyzOnlyForN2o).multiply(BigDecimal_265).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
            tXinyiCarbonMajorUpdate.setWnclBwqrsN2oCo2(wnclBwqrsN2oCo2);
        }

        //污泥处理-污泥热解碳化或气化过程化石源CO-污泥热解过程排放的CH₄的CO₂当量（t CO₂-eq/月）
        BigDecimal wnclWnrjByhXhl = tXinyiCarbonMajor.getWnclWnrjByhXhl();
        BigDecimal wnclWnrjByhHtbl = tXinyiCarbonMajor.getWnclWnrjByhHtbl();
        BigDecimal wnclWnrjByhCo2 = null;
        if(!Objects.isNull(wnclWnrjByhHtbl) && !Objects.isNull(wnclWnrjByhXhl)){
            wnclWnrjByhCo2 = DecimalUtils.getNLengthHalfUp(wnclWnrjByhXhl.multiply(wnclWnrjByhHtbl).multiply(BigDecimal_0_12).multiply(BigDecimal_44).divide(BigDecimal_12, NUMBER_SCALE_6, RoundingMode.HALF_UP).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP).divide(BigDecimal_100, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
            tXinyiCarbonMajorUpdate.setWnclWnrjByhCo2(wnclWnrjByhCo2);
        }

        //污泥处理-污泥热解碳化或气化过程产生的CH₄和N₂O的CO₂排放当量-污泥热解过程排放的CH₄的CO₂当量（t CO₂-eq/月）
        //污泥处理-污泥热解碳化或气化过程产生的CH₄和N₂O的CO₂排放当量-污泥热解过程排放的N₂O的CO₂当量（t CO₂-eq/月）
        Integer wnclWnrjQtLx = tXinyiCarbonMajor.getWnclWnrjQtLx();
        BigDecimal wnclWnrjQtXhl = tXinyiCarbonMajor.getWnclWnrjQtXhl();
        BigDecimal wnclWnrjQtPfyz = tXinyiCarbonMajor.getWnclWnrjQtPfyz();
        BigDecimal wnclWnrjQtPfyzCh4 = BigDecimal.ZERO;
        BigDecimal wnclWnrjQtPfyzN2o = BigDecimal.ZERO;
        BigDecimal wnclWnrjQtCh4Co2 = null;
        BigDecimal wnclWnrjQtN2oCo2 = null;
        if(!Objects.isNull(wnclWnrjQtXhl)){
            if(Objects.isNull(wnclWnrjQtPfyz)){
                if(Objects.isNull(wnclWnrjQtLx)){
                    wnclWnrjQtPfyzCh4 = BigDecimal.ZERO;
                    wnclWnrjQtPfyzN2o = BigDecimal.ZERO;
                }else{
                    if(0 == wnclWnrjQtLx){
                        wnclWnrjQtPfyzCh4 = BigDecimal_0_0581;
                        wnclWnrjQtPfyzN2o = BigDecimal_0_1740;
                    }else if (1 == wnclWnrjQtLx) {
                        wnclWnrjQtPfyzCh4 = BigDecimal_0_0970;
                        wnclWnrjQtPfyzN2o = BigDecimal_0_0580;
                    }else if (2 == wnclWnrjQtLx) {
                        wnclWnrjQtPfyzCh4 = BigDecimal_0_0540;
                        wnclWnrjQtPfyzN2o = BigDecimal_0_0838;
                    }else {
                        wnclWnrjQtPfyzCh4 = BigDecimal.ZERO;
                        wnclWnrjQtPfyzN2o = BigDecimal.ZERO;
                    }
                }
            }
            wnclWnrjQtCh4Co2 = DecimalUtils.getNLengthHalfUp(wnclWnrjQtXhl.multiply(wnclWnrjQtPfyzCh4).multiply(BigDecimal_28).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
            tXinyiCarbonMajorUpdate.setWnclWnrjQtCh4Co2(wnclWnrjQtCh4Co2);
            wnclWnrjQtN2oCo2 = DecimalUtils.getNLengthHalfUp(wnclWnrjQtXhl.multiply(wnclWnrjQtPfyzN2o).multiply(BigDecimal_265).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
            tXinyiCarbonMajorUpdate.setWnclWnrjQtN2oCo2(wnclWnrjQtN2oCo2);
        }

        //处理 污泥处理碳排量 总的碳排放
        if(!Objects.isNull(wnclYyzqCo2)){
            wnclTotalCo2 = wnclTotalCo2.add(wnclYyzqCo2);
        }
        if(!Objects.isNull(wnclDdclCo2)){
            wnclTotalCo2 = wnclTotalCo2.add(wnclDdclCo2);
        }
        if(!Objects.isNull(wnclHyfjCh4Co2)){
            wnclTotalCo2 = wnclTotalCo2.add(wnclHyfjCh4Co2);
        }
        if(!Objects.isNull(wnclHyfjN2oCo2)){
            wnclTotalCo2 = wnclTotalCo2.add(wnclHyfjN2oCo2);
        }
        if(!Objects.isNull(wnclWnfsCo2)){
            wnclTotalCo2 = wnclTotalCo2.add(wnclWnfsCo2);
        }
        if(!Objects.isNull(wnclBwqrsCh4Co2)){
            wnclTotalCo2 = wnclTotalCo2.add(wnclBwqrsCh4Co2);
        }
        if(!Objects.isNull(wnclBwqrsN2oCo2)){
            wnclTotalCo2 = wnclTotalCo2.add(wnclBwqrsN2oCo2);
        }
        if(!Objects.isNull(wnclWnrjByhCo2)){
            wnclTotalCo2 = wnclTotalCo2.add(wnclWnrjByhCo2);
        }
        if(!Objects.isNull(wnclWnrjQtCh4Co2)){
            wnclTotalCo2 = wnclTotalCo2.add(wnclWnrjQtCh4Co2);
        }
        if(!Objects.isNull(wnclWnrjQtN2oCo2)){
            wnclTotalCo2 = wnclTotalCo2.add(wnclWnrjQtN2oCo2);
        }
        //总量
        tXinyiCarbonMajorUpdate.setWnclTotalCo2(BigDecimal.ZERO.compareTo(wnclTotalCo2) == 0 ? null : wnclTotalCo2);

        //------------替碳、碳汇------------
        BigDecimal ttthTotalCo2 = BigDecimal.ZERO;
        //替碳、碳汇-光伏-替碳量-光伏发电替碳量（t CO₂-eq/月）
        BigDecimal thGfZhdl = tXinyiCarbonMajor.getThGfZhdl();
        BigDecimal thGfCo2 = null;
        if(!Objects.isNull(thGfZhdl) && !Objects.isNull(yz)){
            thGfCo2 = DecimalUtils.getNLengthHalfUp(thGfZhdl.multiply(yz).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
            tXinyiCarbonMajorUpdate.setThGfCo2(thGfCo2);
        }

        //替碳、碳汇-再生水回用的替碳量（宜按替代的商业供水的取水、生产和输送的碳排放量计算）-回用水替代供水的替碳量（t CO₂-eq/月
        Integer thZssScgm = tXinyiCarbonMajor.getThZssScgm();
        BigDecimal thZssGyl = tXinyiCarbonMajor.getThZssGyl();
        BigDecimal thZssCo2 = null;
        if(!Objects.isNull(thZssScgm) && !Objects.isNull(thZssGyl) && !Objects.isNull(yz)){
            BigDecimal thZssCo2Yz = BigDecimal.ZERO;
            if(thZssScgm == 0){
                thZssCo2Yz = BigDecimal_0_52;
            }else if(thZssScgm == 1){
                thZssCo2Yz = BigDecimal_0_41;
            }else if(thZssScgm == 2){
                thZssCo2Yz = BigDecimal_0_3;
            }else {
                thZssCo2Yz = BigDecimal.ZERO;
            }
            thZssCo2 = DecimalUtils.getNLengthHalfUp(((BigDecimal_0_2.add(BigDecimal_0_5).subtract(BigDecimal_0_5)).abs().multiply(yz).add(thZssCo2Yz)).multiply(thZssGyl).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
            tXinyiCarbonMajorUpdate.setThZssCo2(thZssCo2);
        }

        //替碳、碳汇-污泥经过厌氧、好氧或碳化处理后，产生的替碳量-污泥产物土地利用的替碳量（t CO₂-eq/月）
        BigDecimal thWnClhFlwnl = tXinyiCarbonMajor.getThWnClhFlwnl();
        BigDecimal thWnClhCo2 = null;
        if(!Objects.isNull(thWnClhFlwnl)){
            thWnClhCo2 = DecimalUtils.getNLengthHalfUp(((BigDecimal_0_03.multiply(BigDecimal_0_61).multiply(BigDecimal_2_116)).add((BigDecimal_0_006.multiply(BigDecimal_0_7).multiply(BigDecimal_1_45)))).multiply(thWnClhFlwnl).divide(BigDecimal_1000, NUMBER_SCALE_6, RoundingMode.HALF_UP), INT_2);
            tXinyiCarbonMajorUpdate.setThWnClhCo2(thWnClhCo2);
        }
        //替碳、碳汇替总碳量
        if(!Objects.isNull(thGfCo2)){
            ttthTotalCo2 = ttthTotalCo2.add(thGfCo2);
        }
        if(!Objects.isNull(thZssCo2)){
            ttthTotalCo2 = ttthTotalCo2.add(thZssCo2);
        }
        if(!Objects.isNull(thWnClhCo2)){
            ttthTotalCo2 = ttthTotalCo2.add(thWnClhCo2);
        }
        //处理一对多的数据
        BigDecimal rbjsTTL = tXinyiCarbonMajorUpdate.getRbjsTTL();
        BigDecimal wnyyxhzqlyTTL = tXinyiCarbonMajorUpdate.getWnyyxhzqlyTTL();
        BigDecimal wnfshrjnlyTTL = tXinyiCarbonMajorUpdate.getWnfshrjnlyTTL();
        if(!Objects.isNull(rbjsTTL)){
            ttthTotalCo2 = ttthTotalCo2.add(rbjsTTL);
        }
        if(!Objects.isNull(wnyyxhzqlyTTL)){
            ttthTotalCo2 = ttthTotalCo2.add(wnyyxhzqlyTTL);
        }
        if(!Objects.isNull(wnfshrjnlyTTL)){
            ttthTotalCo2 = ttthTotalCo2.add(wnfshrjnlyTTL);
        }
        tXinyiCarbonMajorUpdate.setTtthTotalCo2(BigDecimal.ZERO.compareTo(ttthTotalCo2) == 0 ? null : ttthTotalCo2);

        //计算各个大类和总的co2
        BigDecimal ztTotalCo2 = wsclTotalCo2.add(nyyjTotalCo2).add(wnclTotalCo2).subtract(ttthTotalCo2);
        if(ztTotalCo2.compareTo(BigDecimal.ZERO) != 0){//这里计算取了个巧，默认它是0，否则需要多很多判断
            //2024年09月12日17:20:53 这里要取整数
            tXinyiCarbonMajorUpdate.setZtTotalCo2(DecimalUtils.getNLengthHalfUp(ztTotalCo2, INT_0));
        }

        //处理百分比信息
        if(nyyjTotalCo2.compareTo(BigDecimal.ZERO) > 0){//必须是正数
            if(!Objects.isNull(nyyjDlxhZhdlCo2)){
                tXinyiCarbonMajorUpdate.setDlxhPercent(nyyjDlxhZhdlCo2.divide(nyyjTotalCo2, NUMBER_SCALE_6, RoundingMode.HALF_UP));
            }
            if(!Objects.isNull(wscljdyjxhCo2)){
                tXinyiCarbonMajorUpdate.setYjxhPercent(wscljdyjxhCo2.divide(nyyjTotalCo2, NUMBER_SCALE_6, RoundingMode.HALF_UP));
            }
            if(!Objects.isNull(nyyjGrrlCo2)){
                tXinyiCarbonMajorUpdate.setJgrrlPercent(nyyjGrrlCo2.divide(nyyjTotalCo2, NUMBER_SCALE_6, RoundingMode.HALF_UP));
            }
            if(!Objects.isNull(gdsnyrlCo2)){
                tXinyiCarbonMajorUpdate.setRlxhPercent(gdsnyrlCo2.divide(nyyjTotalCo2, NUMBER_SCALE_6, RoundingMode.HALF_UP));
            }
        }

        //处理本月份的 污水厂行业碳排放强度 和 污水厂行业碳排放强度-是否高于同行业平均水平
        BigDecimal avgJsl = this.tXinyiCarbonMajorMapper.selectSumJslByMonth(addYearMonth.replaceAll("-", "/"));
        if(!Objects.isNull(avgJsl)){
            //2024年08月27日09:36:14 计算公式有调整：再乘以10的三次方。
            BigDecimal wschytpfqd = ztTotalCo2.divide(avgJsl, NUMBER_SCALE_10, RoundingMode.HALF_UP).multiply(BigDecimal_1000);
            tXinyiCarbonMajorUpdate.setWschytpfqd(DecimalUtils.getNLengthHalfUp(wschytpfqd, NUMBER_SCALE_2));
            BigDecimal avgCod = this.tXinyiCarbonMajorMapper.selectAvgCodByMonth(addYearMonth.replaceAll("-", "/"));
            //从字典获取不同cod对应的标准
            if(!Objects.isNull(avgCod)){
                List<SysDictData> sysDictData = this.sysDictDataMapper.selectDictDataByType(TPFQD);
                if(!CollectionUtils.isEmpty(sysDictData)){
                    for (SysDictData sysDictDatum : sysDictData) {
                        String dictLabel = sysDictDatum.getDictLabel();
                        if(StringUtils.isNotBlank(dictLabel) && dictLabel.contains(SPLIT_)){
                            String[] split = dictLabel.split(SPLIT_);
                            BigDecimal first = new BigDecimal(split[0]);
                            BigDecimal last = new BigDecimal(split[1]);
                            if(avgCod.compareTo(first) >= 0 && avgCod.compareTo(last) < 0){
                                BigDecimal bzz = new BigDecimal(sysDictDatum.getDictValue());
                                log.info("当前月份平均cod是{},匹配到的标准值是{}", avgCod, bzz);
                                tXinyiCarbonMajorUpdate.setExceedAvgHY(wschytpfqd.compareTo(bzz));
                            }
                        }
                    }
                }
            }
        }
        //更新
        this.tXinyiCarbonMajorMapper.updateTXinyiCarbonMajor(tXinyiCarbonMajorUpdate);
        //处理其他字段(能源药剂百分比)
        return i;
    }

    /**
     * 明细字段的计算处理
     * 类型放入到值集中，然后再判断
     *
     * @param tXinyiCarbonExtra
     * @param yz
     * @return
     */
    private BigDecimal calculate(TXinyiCarbonExtra tXinyiCarbonExtra, BigDecimal yz) {
        int type = tXinyiCarbonExtra.getType();
        BigDecimal amount = tXinyiCarbonExtra.getAmount();
        BigDecimal lyl = tXinyiCarbonExtra.getLyl();
        BigDecimal hz = tXinyiCarbonExtra.getHz();
        SysDictData sysDictData = this.sysDictDataMapper.selectDictDataById(tXinyiCarbonExtra.getDictCode());
        if(Objects.isNull(sysDictData)){
            throw new ServiceException("没有查询到对应的配置项");
        }
        //2024年08月30日14:08:19 计算公式 不统一，并且字段不太一致，所以需要TXinyiCarbonExtra表新增2个字段，代表四个含义。
        if(type == BusinessEnum.CarbonExtraTeypEnum.ZERO.getCode()){
            return DecimalUtils.getNLengthHalfUp(amount.multiply(new BigDecimal(sysDictData.getDictValue()).divide(BigDecimal_1000,NUMBER_SCALE_10, RoundingMode.HALF_UP)), INT_2);
        }else if(type == BusinessEnum.CarbonExtraTeypEnum.ONE.getCode()){
            return DecimalUtils.getNLengthHalfUp(amount.multiply(new BigDecimal(sysDictData.getDictValue()).divide(BigDecimal_1000,NUMBER_SCALE_10, RoundingMode.HALF_UP)), INT_2);
        }else if(type == BusinessEnum.CarbonExtraTeypEnum.TWO.getCode()){
            return DecimalUtils.getNLengthHalfUp(amount.multiply(new BigDecimal(sysDictData.getDictValue()).divide(BigDecimal_1000,NUMBER_SCALE_10, RoundingMode.HALF_UP)), INT_2);
        }else if(type == BusinessEnum.CarbonExtraTeypEnum.THREE.getCode()){
            return DecimalUtils.getNLengthHalfUp(amount.multiply(new BigDecimal(sysDictData.getDictValue()).divide(BigDecimal_1000000000000,NUMBER_SCALE_10, RoundingMode.HALF_UP)), INT_2);
        }else if(type == BusinessEnum.CarbonExtraTeypEnum.FOUR.getCode()){
            return Objects.isNull(yz) ? null : DecimalUtils.getNLengthHalfUp((amount.multiply(yz).add((lyl.multiply(new BigDecimal(sysDictData.getDictValue()))).add((hz.multiply(BigDecimal_0_0022))))).divide(BigDecimal_1000,NUMBER_SCALE_10, RoundingMode.HALF_UP), INT_2);
        }else if(type == BusinessEnum.CarbonExtraTeypEnum.FIVE.getCode()){
            //2024年09月05日13:49:11  公式调整，没有 沼气提纯并网的天然气量/对应焓值 这个参数了
            return Objects.isNull(yz) ? null : DecimalUtils.getNLengthHalfUp((amount.multiply(yz).add((lyl.multiply(new BigDecimal(sysDictData.getDictValue())).divide(BigDecimal_1000,NUMBER_SCALE_10, RoundingMode.HALF_UP)))).divide(BigDecimal_1000,NUMBER_SCALE_10, RoundingMode.HALF_UP), INT_2);
        }else {
            log.error("暂不支持的类型");
            throw new ServiceException("暂不支持的类型");
        }
        //2024年08月28日17:43:58 所有的公式都统一了，都采用下面的方式【感谢郭京京】
//        return amount.multiply(new BigDecimal(sysDictData.getDictValue()).divide(BigDecimal_1000,NUMBER_SCALE_6, RoundingMode.HALF_UP));
    }

    /**
     * 修改碳排放主
     * 
     * @param tXinyiCarbonMajor 碳排放主
     * @return 结果
     */
    @Override
    public int updateTXinyiCarbonMajor(TXinyiCarbonMajor tXinyiCarbonMajor)
    {
        tXinyiCarbonMajor.setUpdateTime(DateUtils.getNowDate());
        return tXinyiCarbonMajorMapper.updateTXinyiCarbonMajor(tXinyiCarbonMajor);
    }

    /**
     * 批量删除碳排放主
     * 
     * @param ids 需要删除的碳排放主主键
     * @return 结果
     */
    @Override
    public int deleteTXinyiCarbonMajorByIds(Long[] ids)
    {
        return tXinyiCarbonMajorMapper.deleteTXinyiCarbonMajorByIds(ids);
    }

    /**
     * 删除碳排放主信息
     * 
     * @param id 碳排放主主键
     * @return 结果
     */
    @Override
    public int deleteTXinyiCarbonMajorById(Long id)
    {
        return tXinyiCarbonMajorMapper.deleteTXinyiCarbonMajorById(id);
    }

    @Override
    public CarbonSmartQueryDB getDBDatasByMonth(String month) {
        return this.tXinyiCarbonMajorMapper.getDBDatasByMonth(month.replaceAll("-", "/"));
    }

    @Override
    public TXinyiCarbonMajor getCarbonDetailsByMonth(String month) {
        TXinyiCarbonMajor tXinyiCarbonMajor = getXinyiCarbonMajorByMonth(month);
        //额外处理其他的关联表：多条数据的
        this.addExtra(tXinyiCarbonMajor);
        return tXinyiCarbonMajor;
    }

    private TXinyiCarbonMajor getXinyiCarbonMajorByMonth(String month) {
        List<TXinyiCarbonMajor> tXinyiCarbonMajors = this.tXinyiCarbonMajorMapper.selectTXinyiCarbonMajorList(TXinyiCarbonMajor.builder().addYearMonth(month).build());
        if(CollectionUtils.isEmpty(tXinyiCarbonMajors))
            throw new ServiceException("未查询到对应的数据，请选择正确的年月");
        if(tXinyiCarbonMajors.size() > 1)
            throw new ServiceException("存在脏数据（重复的数据），对应年月为：" + month);
        return tXinyiCarbonMajors.get(0);
    }


    @Override
    public TXinyiCarbonMajor getCarbonDetailsById(Long id) {
        TXinyiCarbonMajor tXinyiCarbonMajor = this.tXinyiCarbonMajorMapper.selectTXinyiCarbonMajorById(id);
        if(Objects.isNull(tXinyiCarbonMajor))
            throw new ServiceException("未查询到对应的数据，请选择正确的ID");
        //额外处理其他的关联表：多条数据的
        this.addExtra(tXinyiCarbonMajor);
        return tXinyiCarbonMajor;
    }

    private void addExtra(TXinyiCarbonMajor tXinyiCarbonMajor) {
        String addYearMonth = tXinyiCarbonMajor.getAddYearMonth();
        //获取上个月的数据信息
        List<TXinyiCarbonMajor> tXinyiCarbonMajors = this.tXinyiCarbonMajorMapper.selectTXinyiCarbonMajorList(TXinyiCarbonMajor.builder().addYearMonth(DateUtils.getMonthsBefore(addYearMonth, 1)).build());
        if(!CollectionUtils.isEmpty(tXinyiCarbonMajors) && tXinyiCarbonMajors.size() == 1){
            //污水处理碳排放量-环比上个月
            //2024年09月12日16:17:45 修改bug: 1.不要取绝对值  2.当前月和上个月份的值取反了
            TXinyiCarbonMajor tXinyiCarbonMajorLastMonth = tXinyiCarbonMajors.get(0);
            BigDecimal lastMonthCo2 = tXinyiCarbonMajorLastMonth.getWsclTotalCo2();
            BigDecimal nowCo2 = tXinyiCarbonMajor.getWsclTotalCo2();
            if(!Objects.isNull(nowCo2) && !Objects.isNull(lastMonthCo2) && BigDecimal.ZERO.compareTo(lastMonthCo2) != 0){//除数不能为0
                tXinyiCarbonMajor.setWsclTotalCo2Hb(DecimalUtils.getNLengthHalfUp((nowCo2.subtract(lastMonthCo2)).divide(lastMonthCo2, NUMBER_SCALE_10, RoundingMode.HALF_UP).multiply(BigDecimal_100) ,NUMBER_SCALE_2));
            }

            //能源、药剂碳排量-环比上个月
            lastMonthCo2 = tXinyiCarbonMajorLastMonth.getNyyjTotalCo2();
            nowCo2 = tXinyiCarbonMajor.getNyyjTotalCo2();
            if(!Objects.isNull(nowCo2) && !Objects.isNull(lastMonthCo2) && BigDecimal.ZERO.compareTo(lastMonthCo2) != 0){//除数不能为0
                tXinyiCarbonMajor.setNyyjTotalCo2Hb(DecimalUtils.getNLengthHalfUp((nowCo2.subtract(lastMonthCo2)).divide(lastMonthCo2, NUMBER_SCALE_10, RoundingMode.HALF_UP).multiply(BigDecimal_100) ,NUMBER_SCALE_2));
            }

            //污泥处理碳排量-环比上个月
            lastMonthCo2 = tXinyiCarbonMajorLastMonth.getWnclTotalCo2();
            nowCo2 = tXinyiCarbonMajor.getWnclTotalCo2();
            if(!Objects.isNull(nowCo2) && !Objects.isNull(lastMonthCo2) && BigDecimal.ZERO.compareTo(lastMonthCo2) != 0){//除数不能为0
                tXinyiCarbonMajor.setWnclTotalCo2Hb(DecimalUtils.getNLengthHalfUp((nowCo2.subtract(lastMonthCo2)).divide(lastMonthCo2, NUMBER_SCALE_10, RoundingMode.HALF_UP).multiply(BigDecimal_100) ,NUMBER_SCALE_2));
            }

            //替碳、碳汇替碳量-环比上个月
            lastMonthCo2 = tXinyiCarbonMajorLastMonth.getTtthTotalCo2();
            nowCo2 = tXinyiCarbonMajor.getTtthTotalCo2();
            if(!Objects.isNull(nowCo2) && !Objects.isNull(lastMonthCo2) && BigDecimal.ZERO.compareTo(lastMonthCo2) != 0){//除数不能为0
                tXinyiCarbonMajor.setTtthTotalCo2Hb(DecimalUtils.getNLengthHalfUp((nowCo2.subtract(lastMonthCo2)).divide(lastMonthCo2, NUMBER_SCALE_10, RoundingMode.HALF_UP).multiply(BigDecimal_100) ,NUMBER_SCALE_2));
            }

            //总碳排放量-环比上个月
            lastMonthCo2 = tXinyiCarbonMajorLastMonth.getZtTotalCo2();
            nowCo2 = tXinyiCarbonMajor.getZtTotalCo2();
            if(!Objects.isNull(nowCo2) && !Objects.isNull(lastMonthCo2) && BigDecimal.ZERO.compareTo(lastMonthCo2) != 0){//除数不能为0
                tXinyiCarbonMajor.setZtTotalCo2Hb(DecimalUtils.getNLengthHalfUp((nowCo2.subtract(lastMonthCo2)).divide(lastMonthCo2, NUMBER_SCALE_10, RoundingMode.HALF_UP).multiply(BigDecimal_100) ,NUMBER_SCALE_2));
            }
        }
        //最近7个月的数据集合
        TXinyiCarbonMajor tXinyiCarbonMajorReq = new TXinyiCarbonMajor();
        String timeEnd = DateUtils.parseDateToStr(DateUtils.YYYY_MM, DateUtils.getNowDate());
        tXinyiCarbonMajorReq.setHsTimeBegin(DateUtils.getMonthsBefore(timeEnd, INT_7));
        tXinyiCarbonMajorReq.setHsTimeEnd(timeEnd);
        List<TXinyiCarbonMajor> tXinyiCarbonMajorList = this.tXinyiCarbonMajorMapper.selectTXinyiCarbonMajorList(tXinyiCarbonMajorReq);
        List<ChartBasic> recent7PflDatas = new ArrayList<>();
        if(!CollectionUtils.isEmpty(tXinyiCarbonMajorList)){
            for (TXinyiCarbonMajor xinyiCarbonMajor : tXinyiCarbonMajorList) {
                String addYearMonthTmp = xinyiCarbonMajor.getAddYearMonth();
                BigDecimal ztTotalCo2 = xinyiCarbonMajor.getZtTotalCo2();
                ChartBasic chartBasic = new ChartBasic();
                chartBasic.setTime(addYearMonthTmp);
                chartBasic.setZtTotalCo2(ztTotalCo2);
                //获取某个月份的平均进水cod
                BigDecimal avgJsl = this.tXinyiCarbonMajorMapper.selectSumJslByMonth(addYearMonthTmp.replaceAll("-", "/"));
                if(!Objects.isNull(avgJsl) && !Objects.isNull(ztTotalCo2)){
                    chartBasic.setWschytpfqd(DecimalUtils.getNLengthHalfUp(ztTotalCo2.divide(avgJsl, NUMBER_SCALE_10, RoundingMode.HALF_UP).multiply(BigDecimal_1000), NUMBER_SCALE_2));
                }
                recent7PflDatas.add(chartBasic);
            }
        }
        tXinyiCarbonMajor.setRecent7Datas(recent7PflDatas);
        //关联的多条数据
//        tXinyiCarbonMajor.setExtraList();
        tXinyiCarbonMajor.setExtraMap(this.xinyiCarbonExtraMapper.selectTXinyiCarbonExtraList(TXinyiCarbonExtra.builder().majorId(tXinyiCarbonMajor.getId()).build()).stream().collect(Collectors.groupingBy(TXinyiCarbonExtra::getType)));
    }

    public static void main(String[] args) {
        String str = "2024-12";
        System.out.println(str.substring(0, 4));
        System.out.println(str.replace("-", "/"));
        System.out.println(DateUtils.getMonthsBefore(str, 6));
        System.out.println(BigDecimal.ONE.compareTo(BigDecimal_100));
        System.out.println(JSON.toJSONString(null));
        ArrayList<TXinyiCarbonExtra> list = new ArrayList<>();
        list.add(TXinyiCarbonExtra.builder().val(BigDecimal_100).type(1).build());
        list.add(TXinyiCarbonExtra.builder().val(BigDecimal_100).type(0).build());
        list.add(TXinyiCarbonExtra.builder().val(BigDecimal_100).type(0).build());
        Optional<BigDecimal> reduce = list.stream().filter(v -> v.getType() == BusinessEnum.CarbonExtraTeypEnum.ZERO.getCode()).map(TXinyiCarbonExtra::getVal).reduce(BigDecimal::add);
        System.out.println(reduce);
        System.out.println(reduce.isPresent());
        System.out.println(reduce.get());
        list = new ArrayList<>();
        Map<Integer, List<TXinyiCarbonExtra>> collect = list.stream().collect(Collectors.groupingBy(TXinyiCarbonExtra::getType));
        System.out.println(collect);
    }
}
